@@ -1,7 +1,7 @@
 /*
  * =========================================================================
  * dem.c - Routines to handle DEM data.
- * Copyright (c) 2000  Fred M. Erickson
+ * Copyright (c) 2000,2001  Fred M. Erickson
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -475,6 +475,20 @@ process_geo_dem(int dem_fdesc, ssize_t (*read_function)(), struct image_corners
 			}
 
 			*(dem_corners->ptr + j * ONE_DEGREE_DEM_SIZE + i) = strtol((char *)ptr, (char **)&ptr, 10);
+			if (dem_a->elev_units == 1)  {
+				/*
+				 * The main body of drawmap likes to work in meters.
+				 * We satisfy that desire by changing feet into meters
+				 * before passing the data back.  (I think that all GEO
+				 * format DEMs are in meters, but we do the check, just
+				 * to be sure.)
+				 *
+				 * We alter the header information below, after all data
+				 * points have been processed.
+				 */
+				*(dem_corners->ptr + j * ONE_DEGREE_DEM_SIZE + i) =
+					(short)round((double)*(dem_corners->ptr + j * ONE_DEGREE_DEM_SIZE + i) * 0.3048);
+			}
 
 			/*
 			 * If there are less than 1201 south-north profiles, then interpolate to form
@@ -497,6 +511,18 @@ process_geo_dem(int dem_fdesc, ssize_t (*read_function)(), struct image_corners
 		}
 	}
 
+	/*
+	 * The main body of drawmap likes to work in meters.
+	 * We satisfy that desire by changing feet into meters
+	 * before passing the data back.
+	 *
+	 * Here we change the header information.  We already changed the
+	 * actual elevation data above.
+	 */
+	if (dem_a->elev_units == 1)  {
+		dem_a->elev_units = 2;
+	}
+
 	return 0;
 }
 
@@ -549,7 +575,7 @@ process_utm_dem(int dem_fdesc, ssize_t (*read_function)(), struct image_corners
 
 
 	/*
-	 * Make sure that the UTM zone informatio isn't bogus.
+	 * Make sure that the UTM zone information isn't bogus.
 	 */
 	if ((dem_a->zone < 1) || (dem_a->zone > 60))  {
 		fprintf(stderr, "DEM file contains a bad UTM zone (%d).  File ignored.\n", dem_a->zone);
@@ -599,6 +625,10 @@ process_utm_dem(int dem_fdesc, ssize_t (*read_function)(), struct image_corners
 	dem_corners->nw_x_gp = dem_a->nw_x_gp; dem_corners->nw_y_gp = dem_a->nw_y_gp;
 	dem_corners->ne_x_gp = dem_a->ne_x_gp; dem_corners->ne_y_gp = dem_a->ne_y_gp;
 	dem_corners->se_x_gp = dem_a->se_x_gp; dem_corners->se_y_gp = dem_a->se_y_gp;
+	// fprintf(stderr, "%g %g %d      %g %g\n", dem_corners->sw_x_gp, dem_corners->sw_y_gp, dem_a->zone, dem_corners->sw_lat, dem_corners->sw_long);
+	// fprintf(stderr, "%g %g %d      %g %g\n", dem_corners->se_x_gp, dem_corners->se_y_gp, dem_a->zone, dem_corners->se_lat, dem_corners->se_long);
+	// fprintf(stderr, "%g %g %d      %g %g\n", dem_corners->ne_x_gp, dem_corners->ne_y_gp, dem_a->zone, dem_corners->ne_lat, dem_corners->ne_long);
+	// fprintf(stderr, "%g %g %d      %g %g\n", dem_corners->nw_x_gp, dem_corners->nw_y_gp, dem_a->zone, dem_corners->nw_lat, dem_corners->nw_long);
 
 	/*
 	 * If the DEM data don't overlap the image, then ignore them.
@@ -793,7 +823,7 @@ process_utm_dem(int dem_fdesc, ssize_t (*read_function)(), struct image_corners
 				y_gp_max = y_gp;
 //				y_gp_max_loc_i = i; y_gp_max_loc_j = j;
 			}
-			(void)redfearn_inverse(dem_datum, profiles[i].x_gp, profiles[i].y_gp + dem_a->y_res * (double)j, dem_a->zone, &f, &g);
+			(void)redfearn_inverse(dem_datum, x_gp, y_gp, dem_a->zone, &f, &g);
 			if (f < lat_min)  {
 				lat_min = f;
 //				lat_min_loc_i = i; lat_min_loc_j = j;
@@ -863,7 +893,7 @@ process_utm_dem(int dem_fdesc, ssize_t (*read_function)(), struct image_corners
 			y_gp_max = y_gp;
 //			y_gp_max_loc_i = i; x_gp_max_loc_j = -1;
 		}
-		(void)redfearn_inverse(dem_datum, profiles[i].x_gp, profiles[i].y_gp + dem_a->y_res * (double)(profiles[i].num_samples - 1), dem_a->zone, &f, &g);
+		(void)redfearn_inverse(dem_datum, profiles[i].x_gp, y_gp, dem_a->zone, &f, &g);
 		if (f < lat_min)  {
 			lat_min = f;
 //			lat_min_loc_i = i; lat_min_loc_j = profiles[i].num_samples - 1;
@@ -906,7 +936,7 @@ process_utm_dem(int dem_fdesc, ssize_t (*read_function)(), struct image_corners
 				y_gp_max = y_gp;
 //				y_gp_max_loc_i = i; y_gp_max_loc_j = j;
 			}
-			(void)redfearn_inverse(dem_datum, profiles[i].x_gp, profiles[i].y_gp + dem_a->y_res * (double)j, dem_a->zone, &f, &g);
+			(void)redfearn_inverse(dem_datum, x_gp, y_gp, dem_a->zone, &f, &g);
 			if (f < lat_min)  {
 				lat_min = f;
 //				lat_min_loc_i = i; lat_min_loc_j = j;
@@ -961,27 +991,16 @@ process_utm_dem(int dem_fdesc, ssize_t (*read_function)(), struct image_corners
 
 	/*
 	 * Set up the array to be returned to the calling function.
-	 *
-	 * In the past, we used to reset the latitude/longitude
-	 * and x_gp/y_gp corners of the quad to the actual min and
-	 * max values that we just calculated.  This was intended to
-	 * make interpolation more accurate when we inserted this data
-	 * into the image in the main drawmap loop.  However, this made
-	 * only a tiny difference in the map image.  Furthermore,
-	 * if we are going to try to improve accuracy, it is probably
-	 * better to do it by improving the method of interpolation at
-	 * the stage where we copy the DEM data into the image.
-	 * That way, we can localize the details in one place.
-	 * Thus, we leave the four corner locations just as they are in the
-	 * DEM header.  (We filled in these values near the top of this
-	 * function.)
 	 */
+	dem_corners->x_gp_min = x_gp_min;
+	dem_corners->y_gp_min = y_gp_min;
+	dem_corners->x_gp_max = x_gp_max;
+	dem_corners->y_gp_max = y_gp_max;
 	dem_corners->x = dem_size_x;
 	dem_corners->y = dem_size_y;
 
-
-//	fprintf(stderr, "x=%d    y=%d    x_range=%.8g - %.8g    y_range=%.9g - %.9g    lat_range=%.7g - %.7g    long_range=%.8g - %.8g\n",
-//		dem_size_x, dem_size_y, x_gp_min, x_gp_max, y_gp_min, y_gp_max, lat_min, lat_max, long_min, long_max);
+	//fprintf(stderr, "x=%d    y=%d    x_range=%.8g - %.8g    y_range=%.9g - %.9g    lat_range=%.7g - %.7g    long_range=%.8g - %.8g\n",
+	//	dem_size_x, dem_size_y, x_gp_min, x_gp_max, y_gp_min, y_gp_max, lat_min, lat_max, long_min, long_max);
 
 	dem_corners->ptr = (short *)malloc(sizeof(short) * dem_size_x * dem_size_y);
 	if (dem_corners->ptr == (short *)0)  {
@@ -989,6 +1008,7 @@ process_utm_dem(int dem_fdesc, ssize_t (*read_function)(), struct image_corners
 		exit(0);
 	}
 
+
 	/*
 	 * Initialize the new array to HIGHEST_ELEVATION.
 	 */
@@ -1020,11 +1040,49 @@ process_utm_dem(int dem_fdesc, ssize_t (*read_function)(), struct image_corners
 				fprintf(stderr, "FYI:  Overwrite in process_utm_dem at (%d, %d)\n", x, y);
 			}
 			*sptr = profiles[i].data[j];
+			if ((*sptr == 32767) || (*sptr == -32767))  {
+				/*
+				 * Some non-SDTS DEM files appear to mark non-valid data with
+				 * either the value 32767 or -32767.  At least for some files,
+				 * these may represent void_fill and/or edge_fill values, but
+				 * I don't know for sure.  One thing is certain:  they aren't valid
+				 * elevations.  Normally, in drawmap, we convert edge_fill values
+				 * into HIGHEST_ELEVATION, and convert void_fill to an elevation
+				 * of zero.  However, since it isn't clear what these values
+				 * represent, we convert them into HIGHEST_ELEVATION.
+				 * At least that way, the rest of drawmap doesn't have to deal
+				 * with this quirk.
+				 */
+				*sptr = HIGHEST_ELEVATION;
+			}
+			else if (dem_a->elev_units == 1)  {
+				/*
+				 * The main body of drawmap likes to work in meters.
+				 * We satisfy that desire by changing feet into meters
+				 * before passing the data back.
+				 *
+				 * We alter the header information below, after all data
+				 * points have been processed.
+				 */
+				*sptr = (short)round((double)*sptr * 0.3048);
+			}
 		}
 		free(profiles[i].data);
 	}
 	free(profiles);
 
+	/*
+	 * The main body of drawmap likes to work in meters.
+	 * We satisfy that desire by changing feet into meters
+	 * before passing the data back.
+	 *
+	 * Here we change the header information.  We already changed the
+	 * actual elevation data above.
+	 */
+	if (dem_a->elev_units == 1)  {
+		dem_a->elev_units = 2;
+	}
+
 
 
 // For debugging.
