@@ -1,7 +1,7 @@
 /*
  * =========================================================================
  * dem.h - A header file to define parameters for DEM files.
- * Copyright (c) 2000  Fred M. Erickson
+ * Copyright (c) 2000,2001  Fred M. Erickson
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -640,6 +640,11 @@ struct dem_corners  {
 	double se_lat;		// latitude of se corner
 	double se_long;		// longitude of se corner
 
+	double x_gp_min;	// smallest UTM x
+	double y_gp_min;	// smallest UTM y
+	double x_gp_max;	// largest UTM x
+	double y_gp_max;	// largest UTM y
+
 	long x;			// number of samples in a row
 	long y;			// number of samples in a column
 };
