@@ -1,7 +1,7 @@
 /*
  * =========================================================================
  * dem_sdts.c - Routines to handle DEM data.
- * Copyright (c) 2000  Fred M. Erickson
+ * Copyright (c) 2000,2001  Fred M. Erickson
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -132,6 +132,8 @@
  *	SPDM/DMSA/Y   	dem_a->ne_y_gp
  *	SPDM/DMSA/X   	dem_a->se_x_gp
  *	SPDM/DMSA/Y   	dem_a->se_y_gp
+ *	DDOM/DDOM/DVAL	dem_a->void_fill
+ *	DDOM/DDOM/DVAL	dem_a->edge_fill
  *	DDOM/DDOM/DVAL	dem_a->min_elev
  *	DDOM/DDOM/DVAL	dem_a->max_elev
  *	NONE          	dem_a->angle			Old-format field not encoded in SDTS.  Not in use.  Assumed to be always 0.
@@ -742,6 +744,10 @@ parse_dem_sdts(unsigned char *passed_file_name, struct dem_record_type_a *dem_a,
 	need = 4;
 	while (get_subfield(&subfield) != 0)  {
 		if (strcmp(subfield.tag, "DDOM") == 0)  {
+			/*
+			 * Some DEM files use "R" for these values instead of "I".
+			 * Thus, we must check for both.
+			 */
 			if ((strstr(subfield.format, "I") != (char *)0) && (strcmp(subfield.label, "DVAL") == 0))  {
 				save_byte = subfield.value[subfield.length]; subfield.value[subfield.length] = '\0';
 				switch (need)  {
@@ -761,6 +767,25 @@ parse_dem_sdts(unsigned char *passed_file_name, struct dem_record_type_a *dem_a,
 				need--;
 				subfield.value[subfield.length] = save_byte;
 			}
+			else if ((strstr(subfield.format, "R") != (char *)0) && (strcmp(subfield.label, "DVAL") == 0))  {
+				save_byte = subfield.value[subfield.length]; subfield.value[subfield.length] = '\0';
+				switch (need)  {
+				case 4:
+					dem_a->void_fill = round(strtod(subfield.value, (char **)0));
+					break;
+				case 3:
+					dem_a->edge_fill = round(strtod(subfield.value, (char **)0));
+					break;
+				case 2:
+					dem_a->min_elev = round(strtod(subfield.value, (char **)0));
+					break;
+				case 1:
+					dem_a->max_elev = round(strtod(subfield.value, (char **)0));
+					break;
+				}
+				need--;
+				subfield.value[subfield.length] = save_byte;
+			}
 			if (need == 0)  {
 				/* This is all we need.  Break out of the loop. */
 				break;
@@ -1062,6 +1087,26 @@ parse_dem_sdts(unsigned char *passed_file_name, struct dem_record_type_a *dem_a,
 						dem_a->x_gp_first = (double)i;
 					}
 				}
+				/*
+				 * Multiply the UTM coordinates by the scale factors,
+				 * and add in the x and y origins.
+				 * We don't need to do this for the UTM coordinates of the
+				 * map corners because they are stored in real-number format
+				 * rather than binary format.
+				 *
+				 * For some files, x_gp_first and y_gp_first are stored in real-number
+				 * format also.  We handle those files below.
+				 */
+				dem_a->x_gp_first = x_scale_factor * dem_a->x_gp_first + x_origin;
+				/*
+				 * The USGS apparently didn't put in the location of the first
+				 * elevation in the first profile, but rather just put in the location
+				 * of the top left corner of the map area.  Thus, we need to round
+				 * these two values to round multiples of dem_a->x_res and dem_a->y_res.
+				 * The y value must be rounded down and the x value must be rounded up,
+				 * so that the point will be inside the northwest corner of the map area.
+				 */
+				dem_a->x_gp_first = ceil(dem_a->x_gp_first / dem_a->x_res) * dem_a->x_res;
 				need--;
 			}
 			else if ((strstr(subfield.format, "B") != (char *)0) && (strcmp(subfield.label, "Y") == 0))  {
@@ -1086,6 +1131,39 @@ parse_dem_sdts(unsigned char *passed_file_name, struct dem_record_type_a *dem_a,
 						dem_a->y_gp_first = (double)i;
 					}
 				}
+				/*
+				 * Multiply the UTM coordinates by the scale factors,
+				 * and add in the x and y origins.
+				 * We don't need to do this for the UTM coordinates of the
+				 * map corners because they are stored in real-number format
+				 * rather than binary format.
+				 *
+				 * For some files, x_gp_first and y_gp_first are stored in real-number
+				 * format also.  We handle those files below.
+				 */
+				dem_a->y_gp_first = y_scale_factor * dem_a->y_gp_first + y_origin;
+				/*
+				 * The USGS apparently didn't put in the location of the first
+				 * elevation in the first profile, but rather just put in the location
+				 * of the top left corner of the map area.  Thus, we need to round
+				 * these two values to round multiples of dem_a->x_res and dem_a->y_res.
+				 * The y value must be rounded down and the x value must be rounded up,
+				 * so that the point will be inside the northwest corner of the map area.
+				 */
+				dem_a->y_gp_first = floor(dem_a->y_gp_first / dem_a->y_res) * dem_a->y_res;
+				need--;
+			}
+			/*
+			 * In my experience, most files store X and Y as binary values.
+			 * However, a few files use the "R" format, so we also check for it
+			 * here.
+			 */
+			else if ((strstr(subfield.format, "R") != (char *)0) && (strcmp(subfield.label, "X") == 0))  {
+				dem_a->x_gp_first = strtod(subfield.value, (char **)0);
+				need--;
+			}
+			else if ((strstr(subfield.format, "R") != (char *)0) && (strcmp(subfield.label, "Y") == 0))  {
+				dem_a->y_gp_first = strtod(subfield.value, (char **)0);
 				need--;
 			}
 		}
@@ -1115,26 +1193,6 @@ parse_dem_sdts(unsigned char *passed_file_name, struct dem_record_type_a *dem_a,
 	else if (need < 0)  {
 		fprintf(stderr, "Warning:  Got more data from file %s than expected.\n", file_name);
 	}
-	/*
-	 * Multiply the UTM coordinates by the scale factors,
-	 * and add in the x and y origins.
-	 * We don't need to do this for the UTM coordinates of the
-	 * map corners because they are stored in real-number format
-	 * rather than binary format.
-	 */
-	dem_a->x_gp_first = x_scale_factor * dem_a->x_gp_first + x_origin;
-	dem_a->y_gp_first = y_scale_factor * dem_a->y_gp_first + y_origin;
-	/*
-	 * The USGS apparently didn't put in the location of the first
-	 * elevation in the first profile, but rather just put in the location
-	 * of the top left corner of the map area.  Thus, we need to round
-	 * these two values to round multiples of dem_a->x_res and dem_a->y_res.
-	 * The y value must be rounded down and the x value must be rounded up,
-	 * so that the point will be inside the northwest corner of the map area.
-	 */
-	dem_a->x_gp_first = ceil(dem_a->x_gp_first / dem_a->x_res) * dem_a->x_res;
-	dem_a->y_gp_first = floor(dem_a->y_gp_first / dem_a->y_res) * dem_a->y_res;
-
 
 
 	if ((dem_a->horizontal_datum == -1) || (dem_a->horizontal_datum == 1))  {
@@ -1323,6 +1381,10 @@ process_dem_sdts(unsigned char *file_name, struct image_corners *image_corners,
 		struct dem_corners *dem_corners, struct dem_record_type_a *dem_a, struct datum *dem_datum)
 {
 	long i, j, k;
+	union {
+		unsigned long i;
+		float f;
+	} conv;
 	double f, g;
 	long x, y;
 	short s1, s2;
@@ -1364,7 +1426,7 @@ process_dem_sdts(unsigned char *file_name, struct image_corners *image_corners,
 
 
 	/*
-	 * Make sure that the UTM zone informatio isn't bogus.
+	 * Make sure that the UTM zone information isn't bogus.
 	 */
 	if ((dem_a->zone < 1) || (dem_a->zone > 60))  {
 		fprintf(stderr, "DEM file contains a bad UTM zone (%d).  File ignored.\n", dem_a->zone);
@@ -1426,10 +1488,6 @@ process_dem_sdts(unsigned char *file_name, struct image_corners *image_corners,
 	 * Actually, no overlap check is needed, anyway, since the main routine
 	 * will ignore data that is out of bounds.  But we can save a whole
 	 * lot of processing if we can detect out-of-bounds data here.
-	 *
-	 * Because the quads are not rectangular, we might reject a quad that
-	 * slightly overlaps the image, but we won't reject one that overlaps
-	 * by very much.
 	 */
 	if (image_corners->sw_lat < image_corners->ne_lat)  {
 		/* The user has specified image boundaries.  Check for overlap. */
@@ -1482,15 +1540,11 @@ process_dem_sdts(unsigned char *file_name, struct image_corners *image_corners,
 			if ((strstr(subfield.format, "B") != (char *)0) && (strcmp(subfield.label, "ELEVATION") == 0))  {
 				sptr = (dem_corners->ptr + j * dem_size_x + i);
 				/*
-				 * These values, rather than being stored in the expected 'I' format (integer numbers),
+				 * These values, rather than being stored in 'I' format (integer numbers),
 				 * are stored in two's-complement binary.  Thus, they must be properly swabbed
 				 * during conversion to internal form.
 				 */
-				if (subfield.length != 2)  {
-					/* Error */
-					*sptr = HIGHEST_ELEVATION;
-				}
-				else  {
+				if (subfield.length == 2)  {
 					if (byte_order == 0)  {
 						*sptr = (((long)subfield.value[1] << 8) & 0x0000ff00) + ((long)subfield.value[0] & 0x000000ff);
 					}
@@ -1498,14 +1552,81 @@ process_dem_sdts(unsigned char *file_name, struct image_corners *image_corners,
 						*sptr = (((long)subfield.value[0] << 8) & 0x0000ff00) + ((long)subfield.value[1] & 0x000000ff);
 					}
 				}
+				else if (subfield.length == 4)  {
+					/*
+					 * Note:  When the length is 4, we assume that this is a
+					 * BFP32 value, which means that it is a raw binary IEEE 754
+					 * floating point number.  Thus, this conversion won't work
+					 * on machines where IEEE 754 is not the native floating point
+					 * format.  We could convert from binary into the native floating
+					 * point format the hard way, but it appears that most machines
+					 * support IEEE 754, so we will try it this way for a while.
+					 */
+					conv.i = (((long)subfield.value[3] & 0xff) << 24) |
+						  (((long)subfield.value[2] & 0xff) << 16) |
+						  (((long)subfield.value[1] & 0xff) <<  8) |
+						   ((long)subfield.value[0] & 0xff);
+					if (byte_order == 0)  {
+						/* Do nothing. */
+					}
+					else if (byte_order == 1)  {
+						LE_SWAB(&conv.i);
+					}
+					else if (byte_order == 2)  {
+						PDP_SWAB(&conv.i);
+					}
+					*sptr = round(conv.f);
+				}
+				else  {
+					/* Error */
+					*sptr = HIGHEST_ELEVATION;
+				}
 				if (*sptr == dem_a->edge_fill)  {
 					/* This is a point, along the edges of the quad, that doesn't contain valid data. */
 					*sptr = HIGHEST_ELEVATION;
 				}
+				else if (*sptr == 32767)  {
+					/*
+					 * Some DEM files appear to mark invalid data points with 32767.
+					 * I can think of two possible reasons for this; but these are just
+					 * guesses, and the real reason may be entirely different.
+					 * First guess:  it may have been a human data entry error, since the
+					 * edge_fill value is normally -32767.  Second guess:  for a while, it
+					 * may have been the standard was to use 32767 for an edge_fill marker in
+					 * the original (non-SDTS) DEM files; and these values may have been carried
+					 * over as part of the automated conversion process.  I don't know
+					 * how these values got there, but they are clearly not valid elevations.
+					 *
+					 * One concern from all of this is that some of the original DEM files
+					 * appear to contain either of 32767 and -32767 as non-valid data
+					 * markers.  (Perhaps sometimes both, although I haven't located such a
+					 * file yet.)  They may have been automatically carried over into the
+					 * SDTS files during automated conversion.  For the 32767 value, this
+					 * wouldn't appear to be a big problem, because we can still detect it.
+					 * However, the -32767 value is identical to the normal SDTS void_fill
+					 * value.  Thus, unless -32767 meant void_fill in the original DEM files,
+					 * this value may be misinterpreted after the conversion to SDTS.
+					 *
+					 * We treat 32767 in the same way as the edge_fill marker, and convert
+					 * it to HIGHEST_ELEVATION.
+					 */
+					*sptr = HIGHEST_ELEVATION;
+				}
 				else if (*sptr == dem_a->void_fill)  {
 					/* This is a point, somewhere within the quad, that falls within a void in the data. */
 					*sptr = 0;
 				}
+				else if (dem_a->elev_units == 1)  {
+					/*
+					 * The main body of drawmap likes to work in meters.
+					 * We satisfy that desire by changing feet into meters
+					 * before passing the data back.
+					 *
+					 * We alter the header information below, after all data
+					 * points have been processed.
+					 */
+					*sptr = (short)round((double)*sptr * 0.3048);
+				}
 
 				if (i == (dem_size_x - 1))  {
 					break;
@@ -1529,8 +1650,25 @@ process_dem_sdts(unsigned char *file_name, struct image_corners *image_corners,
 	end_ddf();
 
 
+	/*
+	 * The main body of drawmap likes to work in meters.
+	 * We satisfy that desire by changing feet into meters
+	 * before passing the data back.
+	 *
+	 * Here we change the header information.  We already changed the
+	 * actual elevation data above.
+	 */
+	if (dem_a->elev_units == 1)  {
+		dem_a->elev_units = 2;
+	}
+
+
 	dem_corners->x = dem_size_x;
 	dem_corners->y = dem_size_y;
+	dem_corners->x_gp_min = dem_a->x_gp_first;
+	dem_corners->y_gp_min = dem_a->y_gp_first - ((double)dem_size_y - 1.0) * dem_a->y_res;
+	dem_corners->x_gp_max = dem_a->x_gp_first + ((double)dem_size_x - 1.0) * dem_a->x_res;
+	dem_corners->y_gp_max = dem_a->y_gp_first;
 
 // For debugging.
 //	for (i = 0; i < dem_size_x; i++)  {
