@@ -1,7 +1,7 @@
 /*
  * =========================================================================
  * dlg.c - Routines to handle DLG data.
- * Copyright (c) 2000  Fred M. Erickson
+ * Copyright (c) 2000,2001  Fred M. Erickson
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,8 +33,8 @@
 /*
  * Storage for attribute types.
  */
-static long num_A_attrib;
-static long num_L_attrib;
+long num_A_attrib;
+long num_L_attrib;
 struct maj_min attributes_A[MAX_A_ATTRIB];
 struct maj_min attributes_L[MAX_L_ATTRIB];
 
@@ -60,9 +60,9 @@ struct nodes nodes[MAX_NODES];
 struct areas areas[MAX_AREAS];
 struct lines lines[MAX_LINES];
 
-static double lat_se, long_se, lat_sw, long_sw, lat_ne, long_ne, lat_nw, long_nw;
+double lat_se, long_se, lat_sw, long_sw, lat_ne, long_ne, lat_nw, long_nw;
 static double grid_x_se, grid_y_se, grid_x_sw, grid_y_sw, grid_x_ne, grid_y_ne, grid_x_nw, grid_y_nw;
-static long dlg_x_low, dlg_y_low, dlg_x_high, dlg_y_high;
+long dlg_x_low, dlg_y_low, dlg_x_high, dlg_y_high;
 long x_prime;
 
 long utm_zone;
@@ -172,13 +172,12 @@ process_dlg_optional(int fdesc, int gz_flag, struct image_corners *image_corners
 				i++;
 				for (; i < ret_val; i++)  { 
 					if (buf[i] != ' ')  { 
-						goto BEGUINE;
+						break;
 					}
 				}
 				break;
 			}
 		}
-BEGUINE:
 		for (j = i + 1; j < ret_val; j++)  { 
 			/* Sometimes, postal codes are separated by a space, so check for two spaces. */
 			if ((buf[j] == ' ') && (buf[j + 1] == ' '))  {
@@ -885,6 +884,7 @@ BEGUINE:
 			}
 		}
 FIN1:
+		{;}
 	}
 
 	/*
@@ -938,6 +938,7 @@ FIN1:
 				}
 			}
 FIN2:
+			{;}
 		}
 	}
 
