@@ -1,6 +1,6 @@
 /*
  * gunzip - a library allowing programs to directly read gzipped files.
- * Copyright (c) 1997  Fred M. Erickson
+ * Copyright (c) 1997,1998  Fred M. Erickson
  *
  * This software is a stripped-down modification of the standard
  * gzip 1.2.4 distribution.  The original distribution contains
@@ -295,7 +295,7 @@ local int get_method(in)
 		do {c=get_byte();} while (c != 0);
 	    } else {
 		/* Copy the base name. Keep a directory prefix intact. */
-                char *p = basename(ofname);
+                char *p = local_basename(ofname);
                 char *base = p;
 		for (;;) {
 		    *p = (char)get_char();
@@ -1819,7 +1819,7 @@ char *strlwr(s)
  * any version suffix). For systems with file names that are not
  * case sensitive, force the base name to lower case.
  */
-char *basename(fname)
+char *local_basename(fname)
     char *fname;
 {
     char *p;
