@@ -1,7 +1,7 @@
 /*
  * =========================================================================
  * llsearch - A program that extracts a subset of the data in a GNIS file.
- * Copyright (c) 1997  Fred M. Erickson
+ * Copyright (c) 1997,1998,1999,2000  Fred M. Erickson
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * This is a filter to search a GNIS file for any entries within
  * the specified latitude/longitude box.
  *
- * Each GNIS file is for a separate state, but some of the larger
+ * Each US state has its own GNIS file, but some of the larger
  * states span many degrees of latitude and longitude.
  * It is convenient to be able to pull out just the data you need
  * for a given map.
@@ -33,18 +33,31 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include "drawmap.h"
 
-ssize_t buf_read(int, void *, size_t);
-ssize_t buf_write(int, const void *, size_t);
-ssize_t get_a_line(int, void *, size_t);
-double lat_conv(unsigned char *);
-double lon_conv(unsigned char *);
 
+void
+license(void)
+{
+	fprintf(stderr, "This program is free software; you can redistribute it and/or modify\n");
+	fprintf(stderr, "it under the terms of the GNU General Public License as published by\n");
+	fprintf(stderr, "the Free Software Foundation; either version 2, or (at your option)\n");
+	fprintf(stderr, "any later version.\n\n");
+
+	fprintf(stderr, "This program is distributed in the hope that it will be useful,\n");
+	fprintf(stderr, "but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
+	fprintf(stderr, "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
+	fprintf(stderr, "GNU General Public License for more details.\n\n");
+
+	fprintf(stderr, "You should have received a copy of the GNU General Public License\n");
+	fprintf(stderr, "along with this program; if not, write to the Free Software\n");
+	fprintf(stderr, "Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n");
+}
 
 
 main(int argc, char *argv[])
 {
-	unsigned char buf[16384];
+	unsigned char buf[2001];
 	unsigned char *tok_ptr;
 	double latitude_low;
 	double latitude_high;
@@ -56,6 +69,11 @@ main(int argc, char *argv[])
 	int i;
 
 	if (argc != 5)  {
+		if ((argc == 2) && (argv[1][0] == '-') && (argv[1][1] == 'L'))  {
+			license();
+			exit(0);
+		}
+
 		fprintf(stderr, "Usage:  %s latitude_low longitude_low latitude_high longitude_high\n", argv[0]);
 		fprintf(stderr, "        (The latitude/longitude values are in decimal degrees.)\n");
 		fprintf(stderr, "        (West longitude is negative and south latitude is negative.)\n");
@@ -90,45 +108,78 @@ main(int argc, char *argv[])
 		exit(0);
 	}
 
-	while ((ret_val = get_a_line(0, buf, 16383)) > 0)  {
+	while ((ret_val = get_a_line(0, buf, 2000)) > 0)  {
 		buf[ret_val] == '\0';
 
 		/*
 		 * We need to figure out whether it is an old-style or new-style record.
 		 */
-		if ((tok_ptr = strstr(buf, "','")) != (unsigned char *)0)  {
+		if ((tok_ptr = strstr(buf, "\",\"")) != (unsigned char *)0)  {
 			/* New-style record. */
-			tok_ptr += 3;
-			for (i = 0; i < 6; i++)  {
-				if ((tok_ptr = strstr(tok_ptr, "','")) != (unsigned char *)0)  {
-					tok_ptr += 3;
+			if ((tok_ptr + 3) < (buf + ret_val))  {
+				tok_ptr += 3;
+			}
+			else  {
+				fprintf(stderr, "Defective GNIS record:  <%s>\n", buf);
+				continue;
+			}
+			for (i = 0; i < 7; i++)  {
+				if (((tok_ptr = strstr(tok_ptr, "\",\"")) != (unsigned char *)0) && (*tok_ptr != '\0'))  {
+					if ((tok_ptr + 3) < (buf + ret_val))  {
+						tok_ptr += 3;
+					}
+					else  {
+						break;
+					}
 				}
 				else  {
 					break;
 				}
 			}
-			if ((tok_ptr != (unsigned char *)0) && (*tok_ptr != '\0'))  {
-				latitude = atof(tok_ptr);
-				if (((tok_ptr = strstr(tok_ptr, "','")) != (unsigned char *)0) && (*(tok_ptr + 3) != '\0'))  {
-					tok_ptr += 3;
-					longitude = atof(tok_ptr);
-				}
+			if (i != 7)  {
+				/*
+				 * If i != 7, then we ran out of data before finding
+				 * the latitude.  Skip the record.
+				 */
+				fprintf(stderr, "Defective GNIS record:  <%s>\n", buf);
+				continue;
+			}
+			latitude = atof(tok_ptr);
+			if (((tok_ptr = strstr(tok_ptr, "\",\"")) != (unsigned char *)0) && (*tok_ptr != '\0') && (*(tok_ptr + 3) != '\0'))  {
+				tok_ptr += 3;
+				longitude = atof(tok_ptr);
 			}
 			else  {
-				latitude = 0.0;
-				longitude = 0.0;
+				fprintf(stderr, "Defective GNIS record:  <%s>\n", buf);
+				continue;
 			}
 		}
 		else  {
 			/* Old-style record. */
+			if (ret_val < 96)  {
+				/* The record is too short to process.  Ignore it. */
+				fprintf(stderr, "Defective GNIS record (too short):  <%s>\n", buf);
+				continue;
+			}
+
 			/*
 			 * Note:  We assume latitude_low, longitude_low, latitude_high, and longitude_high
 			 * were entered in decimal degrees.
 			 * latitude and longitude from the old-style GNIS files, however are in DDDMMSS format, and
 			 * require special conversion functions.
 			 */
-			latitude = lat_conv(&buf[149]);
-			longitude = lon_conv(&buf[156]);
+			if ((buf[86] != 'N') && (buf[86] != 'S'))  {
+				/* Defective record */
+				fprintf(stderr, "Defective GNIS record (latitude defective):  <%s>\n", buf);
+				continue;
+			}
+			if ((buf[95] != 'E') && (buf[95] != 'W'))  {
+				/* Defective record */
+				fprintf(stderr, "Defective GNIS record (longitude defective):  <%s>\n", buf);
+				continue;
+			}
+			latitude = lat_conv(&buf[80]);
+			longitude = lon_conv(&buf[88]);
 		}
 
 		if ((latitude >= latitude_low) &&
