@@ -1,7 +1,7 @@
 /*
  * =========================================================================
  * sdts2dem.c - A program to convert USGS SDTS DEM files to ordinary USGS DEM files.
- * Copyright (c) 2000  Fred M. Erickson
+ * Copyright (c) 2000,2001  Fred M. Erickson
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -90,6 +90,10 @@ license(void)
 main(int argc, char *argv[])
 {
 	long i, j, k, l, m, n;
+	union {
+		unsigned long i;
+		float f;
+	} conv;
 	int dem_fdesc;
 	int output_fdesc;
 	ssize_t ret_val;
@@ -236,9 +240,12 @@ main(int argc, char *argv[])
 		}
 	}
 	else  {
-		code1 = 'a' + round((fabs(dem_a.se_lat) - floor(fabs(dem_a.se_lat) + 0.01)) * 8.0);
-		code2 = '1' + round((fabs(dem_a.se_long) - floor(fabs(dem_a.se_long) + 0.01)) * 8.0);
-		sprintf(output_file, "%02.2d%03.3d%c%c.dem", (int)fabs(dem_a.se_lat), (int)fabs(dem_a.se_long), code1, code2);
+		code1 = 'a' + floor((fabs(dem_a.se_lat) + (dem_a.se_lat < 0 ? -1.0 : 1.0) * 0.02 - floor(fabs(dem_a.se_lat) + (dem_a.se_lat < 0 ? -1.0 : 1.0) * 0.02)) * 8.0);
+		code2 = '1' + floor((fabs(dem_a.se_long) + (dem_a.se_long < 0 ? 1.0 : -1.0) * 0.02 - floor(fabs(dem_a.se_long) + (dem_a.se_long < 0 ? 1.0 : -1.0) * 0.02)) * 8.0);
+		sprintf(output_file, "%02.2d%03.3d%c%c.dem",
+			(int)(fabs(dem_a.se_lat) + (dem_a.se_lat < 0 ? -1.0 : 1.0) * 0.02),
+			(int)(fabs(dem_a.se_long) + (dem_a.se_long < 0 ? 1.0 : -1.0) * 0.02),
+			code1, code2);
 		if ((output_fdesc = open(output_file, O_WRONLY | O_CREAT | O_EXCL, 0644)) < 0)  {
 			fprintf(stderr, "Can't create %s for writing, errno = %d\n", output_file, errno);
 			exit(0);
@@ -308,11 +315,7 @@ main(int argc, char *argv[])
 				 * are stored in two's-complement binary.  Thus, they must be properly swabbed
 				 * during conversion to internal form.
 				 */
-				if (subfield.length != 2)  {
-					/* Error */
-					*sptr = dem_a.void_fill;
-				}
-				else  {
+				if (subfield.length == 2)  {
 					if (byte_order == 0)  {
 						*sptr = (((long)subfield.value[1] << 8) & 0x0000ff00) + ((long)subfield.value[0] & 0x000000ff);
 					}
@@ -320,6 +323,35 @@ main(int argc, char *argv[])
 						*sptr = (((long)subfield.value[0] << 8) & 0x0000ff00) + ((long)subfield.value[1] & 0x000000ff);
 					}
 				}
+				else if (subfield.length == 4)  {
+					/*
+					 * Note:  When the length is 4, we assume that this is a
+					 * BFP32 value, which means that it is a raw binary IEEE 754
+					 * floating point number.  Thus, this conversion won't work
+					 * on machines where IEEE 754 is not the native floating point
+					 * format.  We could convert from binary into the native floating
+					 * point format the hard way, but it appears that most machines
+					 * support IEEE 754, so we will try it this way for a while.
+					 */
+					conv.i = (((long)subfield.value[3] & 0xff) << 24) |
+						  (((long)subfield.value[2] & 0xff) << 16) |
+						  (((long)subfield.value[1] & 0xff) <<  8) |
+						   ((long)subfield.value[0] & 0xff);
+					if (byte_order == 0)  {
+						/* Do nothing. */
+					}
+					else if (byte_order == 1)  {
+						LE_SWAB(&conv.i);
+					}
+					else if (byte_order == 2)  {
+						PDP_SWAB(&conv.i);
+					}
+					*sptr = round(conv.f);
+				}
+				else  {
+					/* Error */
+					*sptr = dem_a.void_fill;
+				}
 
 				if (i == (dem_a.cols - 1))  {
 					break;
@@ -348,8 +380,10 @@ main(int argc, char *argv[])
 	 * write it out in DEM record format, and in south-to-north,
 	 * west-to-east order.
 	 */
-	x = dem_a.x_gp_first;
+	x = dem_a.x_gp_first - dem_a.x_res;
 	for (j = 0; j < dem_a.cols; j++)  {
+		x = x + dem_a.x_res;
+
 		/*
 		 * Write the profile header.
 		 * First we need to find the minimum and maximum elevations for the profile,
@@ -360,6 +394,33 @@ main(int argc, char *argv[])
 		num_elevs = 0;
 		for (l = 0; l < dem_a.rows; l++)  {
 			sptr = (ptr + l * dem_a.cols + j);
+
+			/*
+			 * Some files contain 32767 as a marker for non-valid
+			 * data.  We take this opportunity to convert these values
+			 * into dem_a.edge_fill so that the rest of the code
+			 * doesn't have to deal with them.
+			 *
+			 * On second thought, one presumes that these values were
+			 * present in the original DEM files, before they were
+			 * converted to SDTS.  Thus, in the spirit of trying to
+			 * produce a DEM file as much like the original as possible,
+			 * we leave the values alone.
+			 *
+			 * There was also the additional problem that eliminating the
+			 * 32767 values would sometimes totally elminate a profile from
+			 * the DEM file, since num_elevs would be zero for a line with
+			 * only 32767 values.  This would mean that the file contents
+			 * would not jibe with the number of lines given in the global
+			 * header at the top of the file.  We could write addtional code
+			 * to deal with this, but it doesn't seem worth it to eliminate
+			 * data values that were probably present in the original source
+			 * file.
+			 */
+			//if (*sptr == 32767)  {
+			//	*sptr = dem_a.edge_fill;
+			//}
+
 			if (*sptr != dem_a.edge_fill)  {
 				num_elevs++;
 
@@ -373,6 +434,9 @@ main(int argc, char *argv[])
 				}
 			}
 		}
+		//if (num_elevs == 0)  {
+		//	continue;
+		//}
 		y = dem_a.y_gp_first - (double)(dem_a.rows - 1) * dem_a.y_res;
 		i = dem_a.rows - 1;
 		while ((i >= 0) && (*(ptr + i * dem_a.cols + j) == dem_a.edge_fill))  {
@@ -419,7 +483,6 @@ main(int argc, char *argv[])
 				exit(0);
 			}
 		}
-		x = x + dem_a.x_res;
 	}
 
 
