@@ -1,7 +1,7 @@
 /*
  * =========================================================================
  * sdts_utils.c - Utility routines for SDTS files.
- * Copyright (c) 2000  Fred M. Erickson
+ * Copyright (c) 2000,2001  Fred M. Erickson
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1347,7 +1347,19 @@ get_subfield(struct subfield *subfield)
 			dr_label++;
 			if (dr_label >= ddr.user[ddr_index].num_labels)  {
 				dr_label = 0;
-				dr_tag++;
+				/*
+				 * I added the following "if" statement, around the "dr_tag++;"
+				 * statement, because some files don't define a sequence of
+				 * X,Y pairs as cartesian arrays (which are handled below), but
+				 * instead define such pairs as a non-cartesian pair of labels that
+				 * gets re-used until the field_limit is reached.  I am somewhat
+				 * doubtful that this type of construct is standards-conforming, but
+				 * the construct is present in some USGS DEM files, so we attempt to
+				 * handle it whether it is conforming or not.
+				 */
+				if (data_index == field_limit)  {
+					dr_tag++;
+				}
 			}
 		}
 		else if (data_index == field_limit)  {
